import usbcreator.install
from usbcreator.misc import fs_size
import logging

def abstract(func):
    def not_implemented(*args):
        raise NotImplementedError('%s is not implemented by the backend.' %
                                  func.__name__)
    return not_implemented

class Backend:
    def __init__(self):
        self.sources = {}
        self.targets = {}
        self.current_source = None
        self.install_thread = None
    
    # Public methods.

    @abstract
    def detect_devices(self):
        pass

    @abstract
    def add_image(self, filename):
        pass

    def set_current_source(self, source):
        if source == None or source in self.sources:
            self.current_source = source
        else:
            raise KeyError, source

    # Signals.

    def source_added_cb(self, drive):
        pass

    def target_added_cb(self, drive):
        pass

    def source_removed_cb(self, drive):
        pass

    def target_removed_cb(self, drive):
        pass
    
    def target_changed_cb(self, udi):
        pass

    # Installation signals.

    def success_cb(self):
        pass

    def failure_cb(self, message=None):
        pass
    
    def install_progress_cb(self, complete, remaining, speed):
        pass

    def install_progress_message_cb(self, message):
        pass

    def retry_cb(self, message):
        pass

    # Internal functions.

    def _update_free(self):
        # XXX evand 2009-07-29: Might be a more general state changed function
        # in the future if we find we care about other properties.  We don't
        # care about mountpoints.

        # This may require pushing the backend into its own thread,
        # unless we keep around the timeout nonsense.

        # TODO evand 2009-07-29: Initialize in Base.__init__().

        if not self.current_source:
            return
        current_source = self.sources[self.current_source]
        keys = self.targets.keys()

        for k in keys:
            free = target['free']
            target['free'] = fs_size(target['device'])[1]
            if free != target['free']:
                changed = True

            target = self.targets[k]
            status = target['status']
            target['status'] = CAN_USE
            if target['capacity'] < current_source['size']:
                target['status'] = CANNOT_USE
            # FIXME evand 2009-07-29: We need to factor in the files we're
            # going to remove and put the adjusted size into a new property, or
            # adjust the free field accordingly.
            elif target['free'] < current_source['size']:
                target['status'] = NEED_SPACE
            if status != target['status']:
                changed = True

            if changed and callable(self.target_changed_cb):
                logging.debug('Source status: %s' % str(target['status']))
                self.target_changed_cb(k)
        
    def _refresh_targets(self):
        pass

    def install(self, source, target, persist, device=None):
        logging.debug('Starting install thread.')
        self.install_thread = usbcreator.install.install(source, target,
                                                         persist,
                                                         device=device)
        # Connect signals.
        self.install_thread.success = self.success_cb
        self.install_thread.failure = self.failure_cb
        self.install_thread.progress = self.install_progress_cb
        self.install_thread.progress_message = self.install_progress_message_cb
        self.install_thread.retry = self.retry_cb
        self.install_thread.start()

    def cancel_install(self):
        if self.install_thread and self.install_thread.is_alive():
            self.install_thread.stop = True
            # TODO evand 2009-07-24: Should set the timeout for join, and
            # continue to process in a loop until the thread finishes, calling
            # into the frontend for UI event processing then break.  The
            # frontend should notify the user that it's canceling by changing
            # the progress message to "Canceling the installation..." and
            # disabiling the cancel button.
            self.install_thread.join()
